package mega.privacy.android.app.myAccount.editProfile

import android.Manifest
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.graphics.*
import android.os.Bundle
import android.view.MenuItem
import android.view.inputmethod.EditorInfo
import androidx.activity.viewModels
import androidx.appcompat.app.AlertDialog
import androidx.core.content.ContextCompat
import androidx.core.view.isVisible
import androidx.core.widget.doAfterTextChanged
import com.google.android.material.appbar.AppBarLayout
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import com.jeremyliao.liveeventbus.LiveEventBus
import mega.privacy.android.app.R
import mega.privacy.android.app.activities.PasscodeActivity
import mega.privacy.android.app.components.AppBarStateChangeListener
import mega.privacy.android.app.components.twemoji.EmojiEditText
import mega.privacy.android.app.constants.EventConstants.EVENT_REFRESH_PHONE_NUMBER
import mega.privacy.android.app.constants.EventConstants.EVENT_USER_EMAIL_UPDATED
import mega.privacy.android.app.constants.EventConstants.EVENT_USER_NAME_UPDATED
import mega.privacy.android.app.databinding.ActivityEditProfileBinding
import mega.privacy.android.app.databinding.DialogChangeEmailBinding
import mega.privacy.android.app.databinding.DialogChangeNameBinding
import mega.privacy.android.app.exportRK.ExportRecoveryKeyActivity
import mega.privacy.android.app.interfaces.SnackbarShower
import mega.privacy.android.app.lollipop.ChangePasswordActivity
import mega.privacy.android.app.modalbottomsheet.ModalBottomSheetUtil.isBottomSheetDialogShown
import mega.privacy.android.app.modalbottomsheet.PhoneNumberBottomSheetDialogFragment
import mega.privacy.android.app.modalbottomsheet.PhotoBottomSheetDialogFragment
import mega.privacy.android.app.myAccount.MyAccountViewModel
import mega.privacy.android.app.myAccount.MyAccountViewModel.Companion.CHECKING_2FA
import mega.privacy.android.app.smsVerification.SMSVerificationActivity
import mega.privacy.android.app.utils.*
import mega.privacy.android.app.utils.AlertDialogUtil.isAlertDialogShown
import mega.privacy.android.app.utils.AlertDialogUtil.quitEditTextError
import mega.privacy.android.app.utils.AlertDialogUtil.setEditTextError
import mega.privacy.android.app.utils.AvatarUtil.getColorAvatar
import mega.privacy.android.app.utils.AvatarUtil.getDominantColor
import mega.privacy.android.app.utils.CacheFolderManager.buildAvatarFile
import mega.privacy.android.app.utils.ChatUtil.StatusIconLocation
import mega.privacy.android.app.utils.ColorUtils.getColorForElevation
import mega.privacy.android.app.utils.Constants.*
import mega.privacy.android.app.utils.Util.*
import mega.privacy.android.app.utils.permission.PermissionUtils.hasPermissions
import nz.mega.sdk.MegaChatApi
import nz.mega.sdk.MegaError
import nz.mega.sdk.MegaError.*
import java.util.*

class EditProfileActivity : PasscodeActivity(), PhotoBottomSheetDialogFragment.PhotoCallback,
    PhoneNumberBottomSheetDialogFragment.PhoneNumberCallback, SnackbarShower {

    companion object {
        private const val PADDING_BOTTOM_APP_BAR = 19F
        private const val PADDING_COLLAPSED_BOTTOM_APP_BAR = 9F
        private const val NAME_SIZE = 16F
        private const val EMAIL_SIZE = 12F
        private const val PADDING_LEFT_STATE = 8F

        private const val CHANGE_NAME_SHOWN = "CHANGE_NAME_SHOWN"
        private const val FIRST_NAME_TYPED = "FIRST_NAME_TYPED"
        private const val LAST_NAME_TYPED = "LAST_NAME_TYPED"
        private const val CHANGE_EMAIL_SHOWN = "CHANGE_EMAIL_SHOWN"
        private const val DELETE_PHOTO_SHOWN = "DELETE_PHOTO_SHOWN"
        private const val EMAIL_TYPED = "EMAIL_TYPED"
        private const val REMOVE_OR_MODIFY_PHONE_SHOWN = "REMOVE_OR_MODIFY_PHONE_SHOWN"
        private const val IS_MODIFY = "IS_MODIFY"
    }

    private val viewModel by viewModels<MyAccountViewModel>()

    private lateinit var binding: ActivityEditProfileBinding

    private var maxWidth = 0
    private var firstLineTextMaxWidthExpanded = 0
    private var stateToolbar = AppBarStateChangeListener.State.IDLE

    private var photoBottomSheet: PhotoBottomSheetDialogFragment? = null

    private var phoneNumberBottomSheetOld: PhoneNumberBottomSheetDialogFragment? = null

    private var isModify = false
    private var removeOrModifyPhoneNumberDialog: AlertDialog? = null
    private var changeNameDialog: AlertDialog? = null
    private var changeEmailDialog: AlertDialog? = null
    private var deletePhotoDialog: AlertDialog? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityEditProfileBinding.inflate(layoutInflater)
        setContentView(binding.root)

        setupView()
        setupObservers()

        if (savedInstanceState != null) {
            if (savedInstanceState.getBoolean(CHANGE_NAME_SHOWN, false)) {
                showChangeNameDialog(
                    savedInstanceState.getString(FIRST_NAME_TYPED),
                    savedInstanceState.getString(LAST_NAME_TYPED)
                )
            }

            if (savedInstanceState.getBoolean(CHANGE_EMAIL_SHOWN, false)) {
                showChangeEmailDialog(savedInstanceState.getString(EMAIL_TYPED))
            }

            if (savedInstanceState.getBoolean(DELETE_PHOTO_SHOWN, false)) {
                deletePhoto()
            }

            if (savedInstanceState.getBoolean(REMOVE_OR_MODIFY_PHONE_SHOWN, false)) {
                showRemovePhoneNumberConfirmation(savedInstanceState.getBoolean(IS_MODIFY, false))
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        if (isAlertDialogShown(changeNameDialog)) {
            outState.putBoolean(CHANGE_NAME_SHOWN, true)
            outState.putString(
                FIRST_NAME_TYPED,
                changeNameDialog?.findViewById<EmojiEditText>(R.id.first_name_field)?.text.toString()
            )

            outState.putString(
                LAST_NAME_TYPED,
                changeNameDialog?.findViewById<EmojiEditText>(R.id.last_name_field)?.text.toString()
            )
        }

        if (isAlertDialogShown(changeEmailDialog)) {
            outState.putBoolean(CHANGE_EMAIL_SHOWN, true)

            outState.putString(
                EMAIL_TYPED,
                changeEmailDialog?.findViewById<EmojiEditText>(R.id.email_field)?.text.toString()
            )
        }

        outState.putBoolean(DELETE_PHOTO_SHOWN, isAlertDialogShown(deletePhotoDialog))

        if (isAlertDialogShown(removeOrModifyPhoneNumberDialog)) {
            outState.putBoolean(REMOVE_OR_MODIFY_PHONE_SHOWN, true)
            outState.putBoolean(IS_MODIFY, isModify)
        }

        super.onSaveInstanceState(outState)
    }

    override fun onDestroy() {
        super.onDestroy()

        changeNameDialog?.dismiss()
        changeEmailDialog?.dismiss()
        deletePhotoDialog?.dismiss()
        removeOrModifyPhoneNumberDialog?.dismiss()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            onBackPressed()
        }

        return super.onOptionsItemSelected(item)
    }

    @Suppress("deprecation") // TODO Migrate to registerForActivityResult()
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        viewModel.manageActivityResult(this, requestCode, resultCode, data, this)
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)

        if (grantResults.isEmpty()) {
            LogUtil.logWarning("Permissions ${permissions[0]} not granted")
            return
        }

        // Take profile picture scenario
        if (requestCode == REQUEST_CAMERA && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            if (hasPermissions(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                checkTakePicture(this, TAKE_PICTURE_PROFILE_CODE)
            } else {
                showSnackbar(StringResourcesUtils.getString(R.string.denied_write_permissions))
            }
        }
    }

    private fun setupView() {
        setUpActionBar()
        setUpAvatar()
        setUpHeader()

        binding.changeName.setOnClickListener { showChangeNameDialog(null, null) }

        binding.addPhoto.setOnClickListener {
            if (isBottomSheetDialogShown(photoBottomSheet))
                return@setOnClickListener

            photoBottomSheet = PhotoBottomSheetDialogFragment()
            photoBottomSheet?.show(supportFragmentManager, photoBottomSheet?.tag)
        }

        binding.changeEmail.setOnClickListener {
            viewModel.check2FA()
            showChangeEmailDialog(null)
        }

        allowNameAndEmailEdition()

        binding.changePassword.setOnClickListener {
            startActivity(Intent(this, ChangePasswordActivity::class.java))
        }

        setupPhoneNumber()

        binding.recoveryKeyButton.setOnClickListener {
            startActivity(Intent(this, ExportRecoveryKeyActivity::class.java))
        }

        binding.logoutButton.setOnClickListener { viewModel.logout(this@EditProfileActivity) }

        setupLogoutWarnings()
    }

    private fun allowNameAndEmailEdition() {
        val permitEditNameAndEmail = !megaApi.isBusinessAccount || megaApi.isMasterBusinessAccount
        binding.changeName.isVisible = permitEditNameAndEmail
        binding.changeNameSeparator.separator.isVisible = permitEditNameAndEmail
        binding.changeEmail.isVisible = permitEditNameAndEmail
        binding.changeEmailSeparator.separator.isVisible = permitEditNameAndEmail
    }

    private fun setupObservers() {
        LiveEventBus.get(EVENT_AVATAR_CHANGE, Boolean::class.java)
            .observe(this) { setUpAvatar() }

        LiveEventBus.get(EVENT_REFRESH_PHONE_NUMBER, Boolean::class.java)
            .observe(this) { setupPhoneNumber() }

        LiveEventBus.get(EVENT_USER_NAME_UPDATED, Boolean::class.java).observe(this) {
            binding.headerLayout.firstLineToolbar.text =
                viewModel.getName().uppercase(Locale.getDefault())
        }

        LiveEventBus.get(EVENT_USER_EMAIL_UPDATED, Boolean::class.java).observe(this) {
            binding.headerLayout.secondLineToolbar.text = viewModel.getEmail()
        }

        viewModel.isProcessingFile().observe(this) { isProcessing ->
            binding.progressBar.isVisible = isProcessing
        }
    }

    /**
     * Shows the result of a name change.
     *
     * @param success True if the name was changed successfully, false otherwise.
     */
    private fun updateName(success: Boolean) {
        binding.progressBar.isVisible = false
        binding.headerLayout.firstLineToolbar.text =
            viewModel.getName().uppercase(Locale.getDefault())

        showSnackbar(
            StringResourcesUtils.getString(
                if (success) R.string.success_changing_user_attributes
                else R.string.error_changing_user_attributes
            )
        )
    }

    /**
     * Shows the result of an email change.
     *
     * @param error Error result of the email change.
     */
    private fun showChangeEmailResult(error: MegaError) {
        binding.progressBar.isVisible = false
        val firstMessageId: Int
        val secondMessageId: Int

        when (error.errorCode) {
            API_OK -> {
                firstMessageId = R.string.email_verification_text_change_mail
                secondMessageId = R.string.email_verification_title
            }
            API_EACCESS -> {
                firstMessageId = R.string.mail_already_used
                secondMessageId = R.string.email_verification_title
            }
            API_EEXIST -> {
                firstMessageId = R.string.mail_changed_confirm_requested
                secondMessageId = R.string.email_verification_title
            }
            else -> {
                firstMessageId = R.string.general_text_error
                secondMessageId = R.string.general_error_word
            }
        }

        showAlert(
            this,
            StringResourcesUtils.getString(firstMessageId),
            StringResourcesUtils.getString(secondMessageId)
        )
    }

    private fun setUpActionBar() {
        setSupportActionBar(binding.headerLayout.toolbar)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.ic_arrow_back_white)
        supportActionBar?.setHomeButtonEnabled(true)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        title = null

        var drawableArrow = ContextCompat.getDrawable(
            applicationContext,
            R.drawable.ic_arrow_back_white
        )

        drawableArrow = drawableArrow?.mutate()

        val statusBarColor =
            getColorForElevation(this, resources.getDimension(R.dimen.toolbar_elevation))

        binding.headerLayout.collapseToolbar.apply {
            if (Util.isDarkMode(this@EditProfileActivity)) {
                setContentScrimColor(statusBarColor)
            }

            setStatusBarScrimColor(statusBarColor)
        }

        val white = ContextCompat.getColor(this, R.color.white_alpha_087)
        val black = ContextCompat.getColor(this, R.color.grey_087_white_087)

        binding.headerLayout.appBar.addOnOffsetChangedListener(object :
            AppBarStateChangeListener() {
            override fun onStateChanged(appBarLayout: AppBarLayout, state: State) {
                stateToolbar = state

                if (state == State.EXPANDED || state == State.COLLAPSED) {
                    val color: Int = if (state == State.EXPANDED) white else black

                    binding.headerLayout.firstLineToolbar.setTextColor(color)
                    binding.headerLayout.secondLineToolbar.setTextColor(color)

                    drawableArrow?.colorFilter =
                        PorterDuffColorFilter(color, PorterDuff.Mode.SRC_IN)

                    supportActionBar?.setHomeAsUpIndicator(drawableArrow)
                    visibilityStateIcon()
                }

                binding.headerLayout.secondLineToolbar.setPadding(
                    0, 0, 0,
                    dp2px(if (state == State.COLLAPSED) PADDING_COLLAPSED_BOTTOM_APP_BAR else PADDING_BOTTOM_APP_BAR)
                )
            }
        })
    }

    /**
     * Checks online status and shows it if needed.
     */
    private fun visibilityStateIcon() {
        val status = megaChatApi.onlineStatus
        val showStatus = stateToolbar == AppBarStateChangeListener.State.EXPANDED
                && (status == MegaChatApi.STATUS_ONLINE || status == MegaChatApi.STATUS_AWAY
                || status == MegaChatApi.STATUS_BUSY || status == MegaChatApi.STATUS_OFFLINE)

        binding.headerLayout.firstLineToolbar.apply {
            maxLines =
                if (showStatus || stateToolbar == AppBarStateChangeListener.State.EXPANDED) 2
                else 1

            if (showStatus) {
                setTrailingIcon(
                    ChatUtil.getIconResourceIdByLocation(
                        this@EditProfileActivity,
                        status,
                        StatusIconLocation.STANDARD
                    ), dp2px(PADDING_LEFT_STATE)
                )
            }

            updateMaxWidthAndIconVisibility(
                if (stateToolbar == AppBarStateChangeListener.State.EXPANDED) firstLineTextMaxWidthExpanded else maxWidth,
                showStatus
            )
        }
    }

    private fun setUpAvatar() {
        val avatar = buildAvatarFile(this, megaApi.myEmail + FileUtil.JPG_EXTENSION)
        var avatarBitmap: Bitmap? = null

        if (avatar != null) {
            avatarBitmap = BitmapFactory.decodeFile(avatar.absolutePath, BitmapFactory.Options())

            if (avatarBitmap != null) {
                binding.headerLayout.toolbarImage.setImageBitmap(avatarBitmap)

                if (!avatarBitmap.isRecycled) {
                    binding.headerLayout.imageLayout.setBackgroundColor(
                        getDominantColor(
                            avatarBitmap
                        )
                    )
                }
            }
        }

        if (avatarBitmap == null) {
            binding.headerLayout.toolbarImage.setImageDrawable(null)
            binding.headerLayout.imageLayout.setBackgroundColor(getColorAvatar(megaApi.myUser))
        }
    }

    private fun setUpHeader() {
        firstLineTextMaxWidthExpanded = outMetrics.widthPixels - dp2px(108f, outMetrics)
        binding.headerLayout.firstLineToolbar.apply {
            setMaxWidthEmojis(firstLineTextMaxWidthExpanded)
            text = viewModel.getName().uppercase(Locale.getDefault())
            textSize = NAME_SIZE
        }

        maxWidth = dp2px(
            if (resources.configuration.orientation == Configuration.ORIENTATION_PORTRAIT) MAX_WIDTH_APPBAR_PORT
            else MAX_WIDTH_APPBAR_LAND,
            outMetrics
        )

        binding.headerLayout.secondLineToolbar.apply {
            maxWidth = maxWidth
            text = viewModel.getEmail()
            textSize = EMAIL_SIZE
        }
    }

    private fun setupPhoneNumber() {
        val alreadyRegistered = viewModel.isAlreadyRegisteredPhoneNumber()

        binding.addPhoneNumber.text = StringResourcesUtils.getString(
            if (alreadyRegistered) R.string.title_modify_phone_number
            else R.string.add_phone_number_action
        )

        binding.addPhoneNumberLayout.apply {
            isVisible = alreadyRegistered || canVoluntaryVerifyPhoneNumber()
            setOnClickListener {
                if (canVoluntaryVerifyPhoneNumber()) {
                    startActivity(
                        Intent(
                            this@EditProfileActivity,
                            SMSVerificationActivity::class.java
                        )
                    )
                } else if (!isBottomSheetDialogShown(phoneNumberBottomSheetOld)) {
                    phoneNumberBottomSheetOld = PhoneNumberBottomSheetDialogFragment()
                    phoneNumberBottomSheetOld!!.show(
                        supportFragmentManager,
                        phoneNumberBottomSheetOld!!.tag
                    )
                }
            }
        }
    }

    /**
     * Checks if there are offline files and transfers.
     * If yes, shows the corresponding warning text at the end. If not, hides the text.
     */
    fun setupLogoutWarnings() {
        val existOfflineFiles = OfflineUtils.existsOffline(this)
        val existOutgoingTransfers = existOngoingTransfers(megaApi)

        binding.logoutWarningText.apply {
            isVisible = existOfflineFiles || existOutgoingTransfers
            text = StringResourcesUtils.getString(
                when {
                    existOfflineFiles && existOutgoingTransfers -> R.string.logout_warning_offline_and_transfers
                    existOfflineFiles -> R.string.logout_warning_offline
                    else -> R.string.logout_warning_transfers
                }
            )
        }
    }

    /**
     * Shows a dialog to change the name of the account.
     *
     * @param firstName Current typed first name if any, null otherwise.
     * @param lastName  Current typed last name if any, null otherwise.
     */
    private fun showChangeNameDialog(firstName: String?, lastName: String?) {
        val dialogBinding = DialogChangeNameBinding.inflate(layoutInflater)

        changeNameDialog = MaterialAlertDialogBuilder(this)
            .setTitle(StringResourcesUtils.getString(R.string.change_name_action))
            .setView(dialogBinding.root)
            .setPositiveButton(StringResourcesUtils.getString(R.string.save_action), null)
            .setNegativeButton(StringResourcesUtils.getString(R.string.button_cancel), null)
            .create()

        changeNameDialog?.apply {
            setOnShowListener {
                quitEditTextError(dialogBinding.firstNameLayout, dialogBinding.firstNameErrorIcon)
                quitEditTextError(dialogBinding.lastNameLayout, dialogBinding.lastNameErrorIcon)

                dialogBinding.firstNameField.apply {
                    setText(firstName ?: viewModel.getFirstName())
                    setOnEditorActionListener { _, actionId, _ ->
                        if (actionId == EditorInfo.IME_ACTION_DONE) {
                            getButton(AlertDialog.BUTTON_POSITIVE).performClick()
                        }

                        false
                    }

                    doAfterTextChanged {
                        quitEditTextError(
                            dialogBinding.firstNameLayout,
                            dialogBinding.firstNameErrorIcon
                        )
                    }
                }

                dialogBinding.lastNameField.apply {
                    setText(lastName ?: viewModel.getLastName())
                    setOnEditorActionListener { _, actionId, _ ->
                        if (actionId == EditorInfo.IME_ACTION_DONE) {
                            getButton(AlertDialog.BUTTON_POSITIVE).performClick()
                        }

                        false
                    }

                    doAfterTextChanged {
                        quitEditTextError(
                            dialogBinding.lastNameLayout,
                            dialogBinding.lastNameErrorIcon
                        )
                    }
                }

                getButton(AlertDialog.BUTTON_POSITIVE).setOnClickListener {
                    var errorShown = false

                    if (dialogBinding.firstNameField.text.isNullOrEmpty()) {
                        errorShown = true
                        setEditTextError(
                            StringResourcesUtils.getString(R.string.error_enter_username),
                            dialogBinding.firstNameLayout, dialogBinding.firstNameErrorIcon
                        )
                    }

                    if (dialogBinding.lastNameField.text.isNullOrEmpty()) {
                        errorShown = true
                        setEditTextError(
                            StringResourcesUtils.getString(R.string.error_enter_userlastname),
                            dialogBinding.lastNameLayout, dialogBinding.lastNameErrorIcon
                        )
                    }

                    if (!errorShown) {
                        binding.progressBar.isVisible =
                            viewModel.changeName(
                                dialogBinding.firstNameField.text.toString(),
                                dialogBinding.lastNameField.text.toString()
                            ) { success -> updateName(success) }

                        dismiss()
                    }
                }
            }

            show()
        }
    }

    /**
     * Shows a dialog to change the email of the account.
     *
     * @param email Current typed email if any, null otherwise.
     */
    private fun showChangeEmailDialog(email: String?) {
        val dialogBinding = DialogChangeEmailBinding.inflate(layoutInflater)

        changeEmailDialog = MaterialAlertDialogBuilder(this)
            .setTitle(StringResourcesUtils.getString(R.string.change_mail_title_last_step))
            .setView(dialogBinding.root)
            .setPositiveButton(StringResourcesUtils.getString(R.string.save_action), null)
            .setNegativeButton(StringResourcesUtils.getString(R.string.button_cancel), null)
            .create()

        changeEmailDialog?.apply {
            setOnShowListener {
                quitEditTextError(dialogBinding.emailLayout, dialogBinding.emailErrorIcon)

                dialogBinding.emailField.apply {
                    setText(email ?: viewModel.getEmail())
                    requestFocus()
                    setSelection(0, text.toString().length)
                    showKeyboardDelayed(this)
                    setOnEditorActionListener { _, actionId, _ ->
                        if (actionId == EditorInfo.IME_ACTION_DONE) {
                            getButton(AlertDialog.BUTTON_POSITIVE).performClick()
                        }

                        false
                    }

                    doAfterTextChanged {
                        quitEditTextError(dialogBinding.emailLayout, dialogBinding.emailErrorIcon)
                    }
                }

                getButton(AlertDialog.BUTTON_POSITIVE).setOnClickListener {
                    if (dialogBinding.emailField.text.isNullOrEmpty()) {
                        setEditTextError(
                            StringResourcesUtils.getString(R.string.error_enter_email),
                            dialogBinding.emailLayout, dialogBinding.emailErrorIcon
                        )
                    } else {
                        val error = viewModel.changeEmail(
                            this@EditProfileActivity,
                            dialogBinding.emailField.text.toString()
                        ) { result -> showChangeEmailResult(result) }

                        binding.progressBar.isVisible = error == null

                        if (error != null && error != CHECKING_2FA) {
                            setEditTextError(
                                error,
                                dialogBinding.emailLayout,
                                dialogBinding.emailErrorIcon
                            )
                        } else {
                            dismiss()
                        }
                    }
                }
            }

            show()
        }
    }

    override fun capturePhoto() {
        viewModel.capturePhoto(this)
    }

    override fun choosePhoto() {
        viewModel.launchChoosePhotoIntent(this)
    }

    override fun deletePhoto() {
        deletePhotoDialog = MaterialAlertDialogBuilder(this)
            .setMessage(R.string.confirmation_delete_avatar)
            .setPositiveButton(R.string.context_delete) { _, _ ->
                viewModel.deleteProfileAvatar(this, this)
            }
            .setNegativeButton(R.string.general_cancel, null)
            .show()
    }

    override fun showRemovePhoneNumberConfirmation(isModify: Boolean) {
        this.isModify = isModify

        val title = StringResourcesUtils.getString(
            if (isModify) R.string.title_modify_phone_number
            else R.string.title_remove_phone_number
        )

        val message = StringResourcesUtils.getString(
            if (isModify) R.string.modify_phone_number_message
            else R.string.remove_phone_number_message
        )

        removeOrModifyPhoneNumberDialog =
            MaterialAlertDialogBuilder(this, R.style.ThemeOverlay_Mega_MaterialAlertDialog)
                .setTitle(title)
                .setMessage(message)
                .setPositiveButton(R.string.general_ok) { _, _ ->
                    viewModel.resetPhoneNumber(isModify, this) {
                        startActivity(Intent(this, SMSVerificationActivity::class.java))
                    }
                }
                .setNegativeButton(StringResourcesUtils.getString(R.string.general_cancel), null)
                .show()
    }

    private fun showSnackbar(message: String) {
        showSnackbar(binding.root, message)
    }

    override fun showSnackbar(type: Int, content: String?, chatId: Long) {
        content?.let { showSnackbar(it) }
    }
}